-- 
-- emsx_top.vhd
--	 ESE MSX-SYSTEM3 / MSX clone on a Cyclone FPGA (ALTERA)
--	 Revision 1.00
-- 
-- Copyright (c) 2006 Kazuhiro Tsujikawa (ESE Artists' factory)
-- All rights reserved.
-- 
-- Redistribution and use of this source code or any derivative works, are 
-- permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, 
--		this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright 
--		notice, this list of conditions and the following disclaimer in the 
--		documentation and/or other materials provided with the distribution.
-- 3. Redistributions may not be sold, nor may they be used in a commercial 
--		product or activity without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-- EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-- PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
-- OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-- WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-- OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
-- ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- 

------------------------------------------------------------------------------
--	NbN
--	modified by t.hara
------------------------------------------------------------------------------
library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_unsigned.all;

entity org_clock_div is
	port(
		clk21m		: in	std_logic;
		o_cpuclk	: out	std_logic;
		o_clkena	: out	std_logic;
		o_clkdiv	: out	std_logic_vector(1 downto 0)
	);
end org_clock_div;

architecture rtl of org_clock_div is
	signal cpuclk	: std_logic := '0';	-- V~[V̕֋XAǉ
	signal clkena	: std_logic := '0';	-- V~[V̕֋XAǉ
	signal clkdiv	: std_logic_vector(1 downto 0) := (others => '0');	-- V~[V̕֋XAǉ
begin
	----------------------------------------------------------------
	-- Clock generator (21.48MHz > 3.58MHz)
	-- pCpuClk should be independent from reset
	----------------------------------------------------------------
	process(clk21m)
		variable jc : std_logic_vector(4 downto 0) := (others => '0');	-- V~[V̕֋XAǉ
	begin
		if (clk21m'event and clk21m = '1') then
			jc(3) := jc(2);
			jc(2) := jc(1);
			jc(1) := jc(0);
			if (jc(3 downto 1) = "010" or jc(3 downto 1) = "101") then
				jc(0) := jc(3);
			else
				jc(0) := not jc(3);
			end if;

			-- CPUCLK : 3.58MHz = 21.48MHz / 6
			if (jc(3 downto 2) = "10") then
				clkena <= '1';
			else
				clkena <= '0';
			end if;
			cpuclk <= jc(3);

			 -- Prescaler : 21.48MHz / 4
			clkdiv <= clkdiv - 1;
		end if;
	end process;

	o_cpuclk <= cpuclk;
	o_clkena <= clkena;
	o_clkdiv <= clkdiv;
end rtl;
